
mtdt = LibStub("AceAddon-3.0"):NewAddon("MeantToDoThat", "AceConsole-3.0", "AceEvent-3.0")

local defaults = {
	global = {
		reciever = nil,
		output = 'smart',
		defList = {
			meant = true,
		},
		custList = {},
		msgList = {},
	},
	char = {
		perChar = 'global',
		reciever = nil,
		output = 'smart',
		defList = {
			meant = true,
		},
		custList = {},
		msgList = {},
	},
}

local options = {
	type = 'group',
	name = "Meant To Do That",
	handler = mtdt,
	get = 'Get',
	set = 'Set',
	args = {
		opt = {
			type = 'keybinding',
			name = 'Options',
			desc = 'Opens options dialog',
			guiHidden = true,
			cmdHidden = false,
			order = 0,
		},
		perChar = {
			type = 'toggle',
			name = 'Settings for character',
			desc = 'Save setting for character, not account',
			get = 'GetChar',
			set = 'SetChar',
			cmdHidden = true,
			order = 10,
		},
		msg = {
			type = 'input',
			name = 'Add death message',
			desc = 'Message to be sent upon death',
			width = "double",
			get = nil,
			set = 'AddMessage',
			cmdHidden = true,
			order = 31,
		},
		custList = {
			type = 'multiselect',
			name = 'Custom message list',
			desc = 'List of custom death messages.',
			width = 'double',
			get = 'GetMulti',
			set = 'SetMulti',
			values = {},
			cmdHidden = true,
			order = 32,
		},
		removeList = {
			type = 'select',
			name = 'Remove custom message',
			desc = 'Choose a custom message to remove then click "remove".',
			get = function (info) return mtdt.remove end,
			set = function (info, value) mtdt.remove = value end,
			values = {},
			cmdHidden = true,
			order = 33,
		},
		removeButton = {
			type = 'execute',
			name = 'Remove',
			desc = 'Removes a selected custom message',
			func = function () mtdt:RemoveMessage() end,
			cmdHidden = true,
			order = 34,
		},
		defList = {
			type = 'multiselect',
			name = 'Default message list',
			desc = 'List of preset death messages.',
			get = 'GetMulti',
			set = 'SetMulti',
			width = 'double',
			cmdHidden = true,
			order = 35,
			values = {
				meant = 'I meant to do that!',
				healer = '%Healer% let me die!',
				tank = '%Tank% didn\'t hold agro!',
			},
		},
		output = {
			type = 'select',
			name = 'Output channel',
			desc = 'Channel to post message.\nSmart group will choose Instance, Raid, or Party depending on what type of group you are in.',
			order = 11,
			values = {
				smart = 'Smart group',
				whisper = 'Whisper',
				instance = 'Instance',
				party = 'Party',
				raid = 'Raid',
				general = 'General',
			},
		},
		reciever = {
			type = 'input',
			name = 'Whisper target',
			desc = 'Player to recieve your message',
			width = "double",
			order  = 21,
		},
	},
}

function mtdt:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("MeantToDoThatDB", defaults)
	-- options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db, true)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Meant To Do That", options)
	self.profilesFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Meant To Do That")
	mtdt:RegisterChatCommand("meanttodothat", "SlashProcessorFunc")
	mtdt:RegisterChatCommand("mtdt", "SlashProcessorFunc")
	mtdt:RegisterEvent("PLAYER_DEAD")
end

function mtdt:OnEnable()
	mtdt:FetchPresence()
	options.args.custList.values = self.db[self.db.char.perChar].msgList
	options.args.removeList.values = self.db[self.db.char.perChar].msgList
end

function mtdt:OnDisable()
	--
end

function mtdt:SlashProcessorFunc( input )
	if input:trim() == "opt" then
		LibStub("AceConfigDialog-3.0"):Open("Meant To Do That")
	elseif input:trim() == "test" then
		mtdt:ChooseMessage()
		mtdt:SendMessage()
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(mtdt, "mtdt", "Meant To Do That", input)
	end
end

function mtdt:Get( info )
	return self.db[self.db.char.perChar][info[#info]]
end

function mtdt:Set( info, value )
	self.db[self.db.char.perChar][info[#info]] = value
	if info[#info] == "reciever" then
		mtdt:FetchPresence()
	end
end

function mtdt:GetMulti( info, key )
	return self.db[self.db.char.perChar][info[#info]][key]
end

function mtdt:SetMulti( info, key, value )
	local temp = self.db[self.db.char.perChar][info[#info]]
	temp[key] = value
	self.db[self.db.char.perChar][info[#info]] = temp
end

function mtdt:GetChar( info )
	return self.db.char.perChar == 'char'
end

function mtdt:SetChar( info, value )
	self.db.char.perChar =  (value and 'char' or 'global')
	options.args.custList.values = self.db[self.db.char.perChar].msgList
	options.args.removeList.values = self.db[self.db.char.perChar].msgList
end

function mtdt:AddMessage( info, value )
	self.db[self.db.char.perChar].msgList[value] = value
	self.db[self.db.char.perChar].custList[value] = true
end

function mtdt:RemoveMessage()
	self.db[self.db.char.perChar].msgList[self.remove] = nil
end

function mtdt:FetchPresence()
	if (self.db[self.db.char.perChar].reciever) then
		self.reciever = GetAutoCompletePresenceID(self.db[self.db.char.perChar].reciever)
	end
end

function mtdt:PLAYER_DEAD()
	mtdt:ChooseMessage()
	mtdt:SendMessage()
end

function mtdt:ChooseMessage()
	local messages = {}
	local messageCount = 0
	self.msg = nil

	for key,value in pairs(self.db[self.db.char.perChar].msgList) do
	    if self.db[self.db.char.perChar].custList[key] then
	    	messages[key] = key
	    	messageCount = messageCount + 1
	    end
	end

	if self.db[self.db.char.perChar].defList.meant then
		messages['I meant to do that!'] = "I meant to do that!"
		messageCount = messageCount + 1
	end

	if self.db[self.db.char.perChar].defList.healer then
		local healer = mtdt:prepHealer()
		if healer then
			messages['healer'] = healer .. " let me die!"
			messageCount = messageCount + 1
		end
	end

	if self.db[self.db.char.perChar].defList.tank then
		local tank = mtdt:prepTank()
		if tank then
			messages['tank'] = tank .. " didn\'t hold agro!"
			messageCount = messageCount + 1
		end
	end


	if messageCount == 0 then
		return
	end

	messageCount = math.random(messageCount)

	for key,value in pairs(messages) do
	    messageCount = messageCount - 1
	    if messageCount == 0 then
	    	self.msg = value
	    end
	end

	if self.msg == "meant" then
		self.msg = "I meant to do that!"
	elseif self.msg == "healer" then
		self.msg = mtdt:prepHealer()
	elseif self.msg == "tank" then
		self.msg = mtdt:prepTank()
	end
end

function mtdt:prepHealer()
	local groupSize = GetNumGroupMembers()
	local healers = {}
	local healSize = 0
	local partyType = ''

	if groupSize == 0 or 1 then
		return nil
	elseif groupSize < 6 then
		partyType = 'party'
	else
		partyType = 'raid'
	end

	for i = 1, groupSize do
		local role = UnitGroupRolesAssigned(partyType..i)
		if role == 'HEALER' then
			healSize = healSize + 1
			healers[healSize] = GetUnitName(partyType..i)
		end
	end

	if healSize > 0 then
		return healers[math.random(healSize)]
	else
		return nil
	end
end

function mtdt:prepTank()
	local groupSize = GetNumGroupMembers()
	local tanks = {}
	local tankSize = 0
	local partyType = ''

	if groupSize == 0 or 1 then
		return nil
	elseif groupSize < 6 then
		partyType = 'party'
	else
		partyType = 'raid'
	end

	for i = 1, groupSize do
		local role = UnitGroupRolesAssigned(partyType..i)
		if role == 'TANK' then
			tankSize = tankSize + 1
			tanks[tankSize] = GetUnitName(partyType..i)
		end
	end

	if tankSize > 0 then
		return tanks[math.random(tankSize)]
	else
		return nil
	end
end

function mtdt:SendMessage()
	if not self.msg then
		return
	end

	if self.db[self.db.char.perChar].output == "whisper" then
		if self.db[self.db.char.perChar].reciever then
			if (self.reciever) then
				BNSendWhisper(self.reciever, self.msg)
			else
				SendChatMessage(self.msg, WHISPER, nil, self.db[self.db.char.perChar].reciever)
			end
		else
			mtdt:Print("Please set a whisper target!")
		end
	elseif IsInGroup(LE_PARTY_CATEGORY_INSTANCE) and  (self.db[self.db.char.perChar].output == "instance" or self.db[self.db.char.perChar].output == "smart") then
		SendChatMessage(self.msg, 'INSTANCE_CHAT')
	elseif IsInRaid() and  (self.db[self.db.char.perChar].output == "raid" or self.db[self.db.char.perChar].output == "smart") then
		SendChatMessage(self.msg, 'RAID')
	elseif IsInGroup(LE_PARTY_CATEGORY_HOME) and  (self.db[self.db.char.perChar].output == "party" or self.db[self.db.char.perChar].output == "smart") then
		SendChatMessage(self.msg, 'PARTY')
	elseif self.db[self.db.char.perChar].output == "general" then
		SendChatMessage(self.msg, 'CHANNEL', nil, 1)
	else
		mtdt:Print("No suitable channel found!")
	end
end